/*!
GPII Canopy Matchmaker

Copyright 2012 OCAD University
Copyright 2012 Antranig Basman

Licensed under the New BSD license. You may not use this file except in
compliance with this License.

You may obtain a copy of the License at
https://github.com/gpii/universal/LICENSE.txt
*/

var fluid = fluid || require("infusion");

var matchMaker = fluid.registerNamespace("gpii.matchMaker");
var canopy = fluid.registerNamespace("gpii.matchMaker.canopy");

canopy.computeFitness = function (leaves, solution) {
    var vector = [];
    for (var i = 0; i < leaves.length; ++ i) {
        var leaf = leaves[i];
        vector[i] = matchMaker.prefixLength(leaf, solution);
    }
    vector = vector.sort(matchMaker.sortDescending);
    return vector;
};

canopy.compareFitness = function (solA, solB) {
    var i = 0, fitA = solA.fitness, fitB = solB.fitness;;
    for (; ; ++i) {
        var endA = i === fitA.length, endB = i === fitB.length;
        if (endA || endB) {
            return endA && endB? 0 : (endB? -1 : 1);
        }
        var diff = fitB[i] - fitA[i];
        if (diff !== 0) {
            return diff;
        }
    }
};

canopy.sortSolutions = function (solutions) {
    return solutions.sort(canopy.compareFitness);
};

canopy.bestCoverage = function (leaves, soln) {
    var best = Number.NEGATIVE_INFINITY;
    for (var i = 0; i < leaves.length; ++ i) {
        var leaf = leaves[i];
        var good = matchMaker.prefixLength(leaf, soln.solution);
        best = Math.max(good, best);
    }
    return best;
};



canopy.rankSolutions = function (leaves, solrecs) {
    fluid.each(solrecs, function (solrec) {
        solrec.fitness = canopy.computeFitness(leaves, solrec.skeleton);
    });
    solrecs = canopy.sortSolutions(solrecs);
    return solrecs;
};

canopy.disposeStrategy = function (leaves, solrecs) {
    var ranked = canopy.rankSolutions(leaves, solrecs);
    var disposed = canopy.disposeSolutionsImpl(leaves, ranked);
    return disposed;
};

canopy.disposeSolutionsImpl = function (leaves, solrecs) {
    var canopy = fluid.transform(leaves, function() {
        return Number.NEGATIVE_INFINITY;
    });

    for (var i = 0; i < solrecs.length; ++ i) {
        var sol = solrecs[i];
        var inCanopy = false;
        for (var j = 0; j < leaves.length; ++ j) {
            var depth = matchMaker.prefixLength(leaves[j], sol.skeleton);
            if (depth > canopy[j]) {
                inCanopy = true;
                canopy[j] = depth;
            }
        }
        if (inCanopy) {
            solrecs[i].disposition = "accept";
        }
        else {
            solrecs[i].disposition = "reject";
        }
    }
    return solrecs;
};
