/*!
GPII Match Maker POST Handler

Copyright 2012 OCAD University

Licensed under the New BSD license. You may not use this file except in
compliance with this License.

You may obtain a copy of the License at
https://github.com/gpii/universal/LICENSE.txt
*/

(function () {

    "use strict";

    var fluid = require("infusion"),
        request = fluid.registerNamespace("gpii.request.matchMaker"),
        kettle = fluid.require("kettle", require);

    request.match = function (matchMaker, when, solutions, preferences) {
        var transform = matchMaker.transformer.transformSettings,
            strategy = fluid.getGlobalValue(matchMaker.options.strategy);

        when(matchMaker.match(preferences, solutions, strategy), function (matchedSolutions) {
            return transform({
                solutions: matchedSolutions,
                preferences: preferences
            });
        });
    };

    request.getSolutions = function (matchMaker, event, model) {
        var device = model.device,
            os = fluid.get(device, "OS.id"),
            version = fluid.get(device, "OS.version");
        matchMaker.solutionsReporter.get({
            os: os,
            version: version
        }, function (solutions) {
            matchMaker.prefilterSolutions(solutions, os, version);
            solutions = matchMaker.prefilterSolutions(solutions, device);
            event.fire(solutions);
        });
    };

    request.transformPreferences = function (matchMaker, event, model) {
        matchMaker.ontologyServer.set(null, {
            preferences: model.preferences
        }, function (preferences) {
            event.fire(preferences);
        });
    };

    request.matchPost = function (that, model) {
        that.events.onMatchRequest.fire(model);
    };

    fluid.defaults("kettle.requests.request.handler.matchPost", {
        gradeNames: ["autoInit", "fluid.eventedComponent"],
        components: {
            "matchMaker": "{gpii.matchMaker}"
        },
        invokers: {
            handle: {
                funcName: "gpii.request.matchMaker.matchPost",
                args: ["{that}", "{request}.req.body"],
                dynamic: true
            },
            match: {
                funcName: "gpii.request.matchMaker.match",
                args: ["{gpii.matchMaker}", "{that}.when", "{arguments}.0", "{arguments}.1"]
            },
            getSolutions: {
                funcName: "gpii.request.matchMaker.getSolutions",
                args: ["{gpii.matchMaker}", "{that}.events.onSolutionsReporter", "{arguments}.0"]
            },
            transformPreferences: {
                funcName: "gpii.request.matchMaker.transformPreferences",
                args: ["{gpii.matchMaker}", "{that}.events.onOntologyServer", "{arguments}.0"]
            }
        },
        events: {
            onMatchRequest: null,
            onSolutionsReporter: null,
            onOntologyServer: null,
            onReadyToMatch: {
                events: {
                   solutions: "onSolutionsReporter",
                   ontology: "onOntologyServer"
                },
                args: ["{arguments}.solutions.0", "{arguments}.ontology.0"]
            }
        },
        listeners: {
            onMatchRequest: [{
                listener: "{that}.getSolutions"
            }, {
                listener: "{that}.transformPreferences"
            }],
            onReadyToMatch: "{that}.match"
        }
    });

})();