/*!
GPII Canopy MatchMaker Tests

Copyright 2012 OCAD University
Copyright 2012 Raising The Floor - International

Licensed under the New BSD license. You may not use this file except in
compliance with this License.

You may obtain a copy of the License at
https://github.com/gpii/universal/LICENSE.txt
*/

// Declare dependencies
/*global require, fluid, jqUnit, gpii, start*/

var fluid = fluid || require("infusion");
var gpii = fluid.registerNamespace("gpii");

(function () {
    "use strict";

    fluid.registerNamespace("gpii.tests.flatMatchMaker");

    var magnifier = {"settingsHandlers": [{
        "type": "gpii.integrationTesting.mockSettingsHandler",
        "capabilities": ["display.screenEnhancement"],
        "capabilitiesTransformations": {
            "mag-factor": "display.screenEnhancement.magnification",
            "show-cross-hairs": "display.screenEnhancement.-provisional-showCrosshairs",
            "mouse-tracking": {
                "transform": {
                    "type": "fluid.transforms.valueMapper",
                    "inputPath": "display.screenEnhancement.tracking",
                    "options": {
                        "mouse": {
                            "outputValue": "centered"
                        }
                    }
                }
            }
        }
    }]
    };

    var lesserMagnifier = {"settingsHandlers": [{
        "type": "gpii.integrationTesting.mockSettingsHandler",
        "capabilitiesTransformations": {
            "mag-factor": "display.screenEnhancement.magnification"
        }
    }]
    };

    var sammyProfile = {
        "display": {
            "screenEnhancement": {
                "fontSize": 24,
                "foregroundColor": "white",
                "backgroundColor": "black",
                "fontFace": {
                    "fontName": ["Comic Sans"],
                    "genericFontFace": "sans serif"
                },
                "magnification": 2.0,
                "tracking": ["mouse"],
                "invertImages": true,
                "-provisional-showCrosshairs": true
            }
        }
    };

    var carlaBit = {
        "content": {
        "adaptationPreference": [
            {
                "adaptationType": "caption",
                "language": "en"
            },
            {}
            ]
        }
    };

    var carlaBit2 = {
      "display": {
      "applications": [{
          "name": "UI Options",
          "id": "fluid.uiOptions.windows"
          }]
       }
    };

    var matchMaker = gpii.matchMaker;

    var runFlat = function(profile, solutions) {
        return matchMaker.disposeSolutions(profile, solutions, matchMaker.flat.disposeStrategy);
    };

    gpii.tests.flatMatchMaker.runTests = function() {

        jqUnit.module("MatchMaker");
        jqUnit.test("Rank and dispose solutions", function() {
            runFlat(sammyProfile, [magnifier]).then(function (disposed) {
                jqUnit.assertEquals("Accepted relevant solution", "accept", disposed[0].disposition);
            });
            runFlat(carlaBit, [magnifier]).then(function (disposed) {
                jqUnit.assertEquals("Rejected irrelevant solution", "reject", disposed[0].disposition);
            });
            runFlat(carlaBit2, [magnifier]).then(function (disposed) {
                jqUnit.assertEquals("Rejected mostly irrelevant solution", "reject", disposed[0].disposition);
            });
        });
    };

}());