/*!
GPII Ontology Server GET Handler

Copyright 2012 OCAD University

Licensed under the New BSD license. You may not use this file except in
compliance with this License.

You may obtain a copy of the License at
https://github.com/gpii/universal/LICENSE.txt
*/

(function () {

    "use strict";

    var fluid = require("infusion"),
        gpii = fluid.registerNamespace("gpii"),
        kettle = fluid.require("kettle", require);

    fluid.defaults("kettle.requests.request.handler.ontologyConvertPost", {
        gradeNames: ["autoInit", "fluid.littleComponent"],
        invokers: {
            handle: {
                funcName: "gpii.ontologyConvertPost",
                args: ["{that}", "{requestProxy}", "{request}.req.params.to", "{request}.req.body.preferences", "{ontologyServer}.ontologySource"],
                dynamic: true
            }
        }
    });

    gpii.ontologyConvertPost = function (that, requestProxy, to, preferences, ontologySource) {
        if (to !== "ISO24751-2") {
            requestProxy.events.onError.fire({
                isError: true,
                message: "Ontology server currently support only transformations to ISO24751-2 ontology."
            });
            return;
        }
        var source = gpii.ontologyServer.parsePreferences(preferences, to);
        if (source.from === source.to) {
            requestProxy.events.onSuccess.fire(preferences);
            return;
        }
        that.when(ontologySource.get({
            name: source.transformation
        }), function (transformations) {
            var transformed;
            transformations = gpii.ontologyServer.expandTransformations(transformations);
            transformed = fluid.model.transformWithRules(preferences, transformations);
            return transformed;
        });
    };

})();